Vamp = LibStub("AceAddon-3.0"):NewAddon("Vamp", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
Vamp.MINOR_VERSION = tonumber(("$Revision: 42 $"):match("%d+"))
local L = LibStub("AceLocale-3.0"):GetLocale("Vamp")
local Vamp = Vamp
------


--[[
      VARIABLES ----------------------------------------------------------------------------------------------------------------
  ]]

local isEnabled = false
local POLLING_FREQUENCY = 1.0
local LIST_SIZE = 30
local guildList = {}
local raid = {}
local vampireDebuff = GetSpellInfo(70867) -- Essence of the Blood Queen
local vampireFrenzyDebuff = GetSpellInfo(71474) -- Frenzied Bloodthirst
local mindControlledDebuff = GetSpellInfo(70923) -- Uncontrollable Frenzy
local testDebuff = nil--58600 -- Restricted Flight Area
Vamp.inCombat = false
local lastRaidWarning = 0



--[[
      UPVALUES -----------------------------------------------------------------------------------------------------------------
  ]]
  
local _G = getfenv(0)
local pairs = _G.pairs
local strlower = _G.strlower
local strupper = _G.strupper



--[[
      UTILITIES ----------------------------------------------------------------------------------------------------------------
  ]]

function Vamp:tcopy(to, from)
 for k, v in pairs(from) do
  if type(v) == "table" then
   to[k] = {}
   Vamp:tcopy(to[k], v)
  else
   to[k] = v
  end
 end
end


local newOrder
do
	local current = 0
	function newOrder()
		current = current + 1
		return current
	end
end


local function colorize(s, color, highlight)
	if color and s then
		if highlight then
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 128 + 127, (color.g or 1) * 128 + 127, (color.b or 1) * 128 + 127, s)
		else
			return string.format("|cff%02x%02x%02x%s|r", (color.r or 1) * 255, (color.g or 1) * 255, (color.b or 1) * 255, s)
		end
	else
		return s
	end
end



--[[
      OPTIONS ------------------------------------------------------------------------------------------------------------------
  ]]

Vamp.options = {
	name = "Vamp         r|cff20ff20"..Vamp.MINOR_VERSION.."|r",
	handler = Vamp,
	type = "group",
	childGroups = "tab",
	args = {		

		choiceMethod = {
			type = "select",
			name = L["Choose players based on"],
			width = "full",
			values = function()
				local numRanks = GuildControlGetNumRanks()
				local t = {}
				t[1] = L["Players in raid only"]
				for i = 1, numRanks do
					t[i + 1] = L["Guild rank: "]..GuildControlGetRankName(i)..L[" (or higher)"]
				end
				return t
			end,
			get = function() return Vamp.db.profile.choiceMethod end,
			set = function(self, val)
				Vamp.db.profile.choiceMethod = val
				LibStub("AceConfigRegistry-3.0"):NotifyChange("Vamp")
			end,
			order = newOrder(),
		}, -- enable

--[[ General ]]----------------------------------------------------------------------------------------------------------------------

		options = {
			type = "group",
			name = L["Options"],
			order = newOrder(),
			childGroups = "tree",
			args = {
			
				enable = {
					type = "toggle",
					name = L["Enable Vamp"],
					get = function() return Vamp.db.profile.enabled end,
					set = function(self, val)
						Vamp.db.profile.enabled = val
						if val then Vamp:Enable() else Vamp:Disable() end
					end,
					order = newOrder(),
				}, -- enable
				
				general = {
					type = "group",
					name = L["General"],
					order = newOrder(),
					args = {
					
						biteUnlisted = {
							order = newOrder(),
							type = "toggle",
							name = L["Bite unlisted players"],
							desc = L["Players in raid groups 1-5, but not listed in any priority list, will be considered valid bite targets. They will be prioritized above the healers."],
							get = function() return Vamp.db.profile.biteUnlisted end,
							set = function(self, val) Vamp.db.profile.biteUnlisted = val end,
						}, -- biteUnlisted
					
					}
				},

				raidWarnings = {
					type = "group",
					name = L["Raid Warnings"],
					order = newOrder(),
					args = {
					
						raidWarn = {
							order = newOrder(),
							type = "toggle",
							name = L["Send raid warnings"],
							desc = L["Sends a raid warning when a bite target dies or disconnects. Warnings will only be sent every several seconds, configured below."],
							get = function() return Vamp.db.profile.raidWarn end,
							set = function(self, val) Vamp.db.profile.raidWarn = val end,
						}, -- raidWarn
						
						raidWarnThrottle = {
							order = newOrder(),
							type = "range",
							name = L["Raid warning throttle"],
							desc = L["Raid warnings about dead or disconnected bite targets will only be sent once every X seconds, configured here. If players die or disconnect during the throttle period, no warning will appear."],
							min = 0,
							max = 360,
							step = 1,
							get = function() return Vamp.db.profile.raidWarnThrottle end,
							set = function(self, val) Vamp.db.profile.raidWarnThrottle = val end,
						}, -- raidWarnThrottle
					
					}
				},
				
			}
		},


--[[ Melee DPS ]]--------------------------------------------------------------------------------------------------------------------

		melee = {
			type = "group",
			name = L["Melee"],
			order = newOrder(),
			args = {
				desc = {
					order = 0,
					type = "description",
					name = format(L["Choose up to %d players who act as melee DPS for the fight. Only players who are actually in your current raid groups 1-5 will be used for bite targets during the fight.\n\nHigher numbered players will be a higher priority for bites. You can skip slots (for example, if a player changes to healing mid-raid, just empty them out of the slot)."], LIST_SIZE),
				},
			}
		},


--[[ Ranged DPS ]]-------------------------------------------------------------------------------------------------------------------

		ranged = {
			type = "group",
			name = L["Ranged"],
			order = newOrder(),
			args = {
				desc = {
					order = 0,
					type = "description",
					name = format(L["Choose up to %d players who act as ranged DPS for the fight. Only players who are actually in your current raid groups 1-5 will be used for bite targets during the fight.\n\nHigher numbered players will be a higher priority for bites. You can skip slots (for example, if a player changes to healing mid-raid, just empty them out of the slot)."], LIST_SIZE),
				},
			}
		},


--[[ Healers ]]----------------------------------------------------------------------------------------------------------------------

		healers = {
			type = "group",
			name = L["Healers"],
			order = newOrder(),
			args = {
				desc = {
					order = 0,
					type = "description",
					name = L["Choose up to 10 players who act as healers for the fight. Only players who are actually in your current raid groups 1-5 will be used for bite targets during the fight.\n\nHigher numbered players will be a higher priority for bites. You can skip slots (for example, if a player changes to DPS mid-raid, just empty them out of the slot)."],
				},
			}
		},


--[[ Tanks ]]------------------------------------------------------------------------------------------------------------------------

		tanks = {
			type = "group",
			name = L["Tanks"],
			order = newOrder(),
			args = {
			
				desc = {
					order = 0,
					type = "description",
					name = L["The off-tank will be chosen once all melee and ranged DPS have been bitten (or died). After that, all healers will be chosen. The main tank is the very last player chosen to be bitten."],
				},

				mainTank = {
					order = newOrder(),
					type = "select",
					name = L["Main tank"],
					values = function() return Vamp:GetUnitsForDropdown("tanks", 1) end,
					get = function() return Vamp.db.profile.tanks[1] end,
					set = function(self, val) Vamp.db.profile.tanks[1] = val end,
				}, -- mainTank
				
				offTank = {
					order = newOrder(),
					type = "select",
					name = L["Off tank"],
					values = function() return Vamp:GetUnitsForDropdown("tanks", 2) end,
					get = function() return Vamp.db.profile.tanks[2] end,
					set = function(self, val) Vamp.db.profile.tanks[2] = val end,
				}, -- offTank
				
			}
		},

	}
}



-- Generate unit options
for i = 1, LIST_SIZE do
	Vamp.options.args.melee.args["melee"..i] = {
		order = i,
		type = "select",
		name = L["Priority "]..tostring(i),
		values = function() return Vamp:GetUnitsForDropdown("melee", i) end,
		get = function() return Vamp.db.profile.melee[i] end,
		set = function(self, val) Vamp.db.profile.melee[i] = val end,
	}
end

for i = 1, LIST_SIZE do
	Vamp.options.args.ranged.args["ranged"..i] = {
		order = i,
		type = "select",
		name = L["Priority "]..tostring(i),
		values = function() return Vamp:GetUnitsForDropdown("ranged", i) end,
		get = function() return Vamp.db.profile.ranged[i] end,
		set = function(self, val) Vamp.db.profile.ranged[i] = val end,
	}
end

for i = 1, 10 do
	Vamp.options.args.healers.args["healers"..i] = {
		order = i,
		type = "select",
		name = L["Priority "]..tostring(i),
		values = function() return Vamp:GetUnitsForDropdown("healers", i) end,
		get = function() return Vamp.db.profile.healers[i] end,
		set = function(self, val) Vamp.db.profile.healers[i] = val end,
	}
end



function Vamp:GetUnitsForDropdown(type, num)
	local t = { [""] = "" }
	if Vamp.db.profile.choiceMethod == 1 then
		if not UnitInRaid("player") then return t end
			local numRaid = GetNumRaidMembers()
			for i = 1, numRaid do
				local playerName, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
				local found = false
				for index, playerName2 in pairs(Vamp.db.profile.melee) do
					if playerName2 == playerName and ((type == "melee" and index ~= num) or type ~= "melee") then
						found = true
						break
					end
				end
				for index, playerName2 in pairs(Vamp.db.profile.ranged) do
					if playerName2 == playerName and ((type == "ranged" and index ~= num) or type ~= "ranged") then
						found = true
						break
					end
				end
				for index, playerName2 in pairs(Vamp.db.profile.healers) do
					if playerName2 == playerName and ((type == "healers" and index ~= num) or type ~= "healers") then
						found = true
						break
					end
				end
				for index, playerName2 in pairs(Vamp.db.profile.tanks) do
					if playerName2 == playerName and ((type == "tanks" and index ~= num) or type ~= "tanks") then
						found = true
						break
					end
				end
				if not found then t[playerName] = colorize(playerName, (RAID_CLASS_COLORS[({UnitClass(playerName)})[2]])) end
			end
			
	else
		for playerName, attributes in pairs(guildList) do
			if (attributes[1] + 1) <= (Vamp.db.profile.choiceMethod - 1) then
				local found = false
				for index, playerName2 in pairs(Vamp.db.profile.melee) do
					if playerName2 == playerName and ((type == "melee" and index ~= num) or type ~= "melee") then
						found = true
						break
					end
				end
				for index, playerName2 in pairs(Vamp.db.profile.ranged) do
					if playerName2 == playerName and ((type == "ranged" and index ~= num) or type ~= "ranged") then
						found = true
						break
					end
				end
				for index, playerName2 in pairs(Vamp.db.profile.healers) do
					if playerName2 == playerName and ((type == "healers" and index ~= num) or type ~= "healers") then
						found = true
						break
					end
				end
				for index, playerName2 in pairs(Vamp.db.profile.tanks) do
					if playerName2 == playerName and ((type == "tanks" and index ~= num) or type ~= "tanks") then
						found = true
						break
					end
				end
				if not found then t[playerName] = colorize(playerName, RAID_CLASS_COLORS[attributes[2]]) end
			end
		end
	end
	return t
end



--[[
      INITIALIZATION -----------------------------------------------------------------------------------------------------------
  ]]

function Vamp:OnInitialize()
	-- Register database
 self.db = LibStub("AceDB-3.0"):New("VampDB", self.defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
		
	-- Register options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Vamp", Vamp.options)
	Vamp.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Vamp", "Vamp")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Vamp-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(Vamp.db))
	Vamp.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Vamp-Profiles", "Profiles", "Vamp")

	-- Register for slash commands
	self:RegisterChatCommand("vamp", "ChatCommand")
	self:RegisterChatCommand("v", "ChatCommand")
end


function Vamp:OnEnable()
	if self.db.profile.enabled then

		-- Register for events
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnEvent")
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnEvent")
		self:RegisterEvent("GUILD_ROSTER_UPDATE", "OnEvent")
		self:RegisterEvent("RAID_ROSTER_UPDATE", "OnEvent")
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnCombatEvent")
		
		-- Update data
		GuildRoster()
		self:UpdateRaid()
		
		-- Turn us on
		isEnabled = true

		--[===[@debug@
			self:Print(L["Loaded"])
		--@end-debug@]===]
		
	end
end


function Vamp:OnDisable()
	-- Unregister all events
	self:UnregisterAllEvents()
	
	-- Cancel all timers
	self:CancelAllTimers()

	-- Turn us off
	isEnabled = false
	
	--[===[@debug@
		self:Print(L["Disabled"])
	--@end-debug@]===]
end


function Vamp:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end


function Vamp:OnProfileChanged(event, database, newProfileKey)
	--[===[@debug@
		self:Print(L["Profile modified, rebooting"])
	--@end-debug@]===]
 self:Disable()
 self:Enable() 
end



--[[
      EVENTS -------------------------------------------------------------------------------------------------------------------
  ]]

function Vamp:OnEvent(event, ...)

	if event == "GUILD_ROSTER_UPDATE" then
		local num = GetNumGuildMembers(true)
		guildList = {}
		for i = 1, num do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName = GetGuildRosterInfo(i)
			if level >= 80 then guildList[name] = { rankIndex, classFileName } end
			--LibStub("AceConfigRegistry-3.0"):NotifyChange("Vamp")
		end

	elseif event == "RAID_ROSTER_UPDATE" then
		self:UpdateRaid()
						
	end


 if isEnabled then
 
		if event == "PLAYER_REGEN_DISABLED" then
			self.inCombat = true
			if UnitInRaid("player") then
				self:ResetFight()
				self:ScheduleRepeatingTimer("OnTimer", POLLING_FREQUENCY)
			end
			
		elseif event == "PLAYER_REGEN_ENABLED" then
			self.inCombat = false
			self:CancelAllTimers()
			self:UpdateRaid()
			
		end
		
 end
end


function Vamp:UpdateRaid()
	if UnitInRaid("player") then
		if not self.inCombat then raid = {} end
		local numRaid = GetNumRaidMembers()
		for i = 1, numRaid do
			local playerName, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
			if not self.inCombat then
				raid[playerName] = {}
				raid[playerName].group = subgroup
				raid[playerName].isVampire = false
				raid[playerName].isBiting = false
				raid[playerName].mindControlled = false
			else
				if not online then
					-- Someone disconnected during combat. Let's see if it's someone we care about
					if raid[playerName] and raid[playerName].group <= 5 then
					
						-- See if the disconnected person was a bite target
						if raid[playerName].isVictim then
							raid[playerName].isVictim = nil
							
							-- Find out who was biting them and give them a new assignment
							for k, v in pairs(raid) do
								if v.yourVictim == playerName then
									self:ReassignTarget(k, playerName, L["disconnected"])
									break
								end
							end
						end -- was a bite victim
						
					end -- was a player in groups 1-5
				end -- disconnected player
			end -- in combat
			
		end -- player loop
	end -- we're in a raid
end



--[[
      FIGHT LOGIC --------------------------------------------------------------------------------------------------------------
  ]]

local marks = {}


function Vamp:ResetFight()
	marks = {
		[1] = "",
		[2] = "",
		[3] = "",
		[4] = "",
		[5] = "",
		[6] = "",
		[7] = "",
		[8] = "",
	}
	local numRaid = GetNumRaidMembers()
	for i = 1, numRaid do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
		SetRaidTarget(name, 0)
	end
end


function Vamp:OnTimer()
	-- Set raid marks
	for mark, player in pairs(marks) do
		if player ~= "" then
			SetRaidTarget(player, mark)
		end
	end
end


function Vamp:ReassignTarget(playerName, deadPlayer, reason)
 local biteTarget = self:GetBiteTarget(playerName)
 if biteTarget then
		raid[playerName].yourVictim = biteTarget
		raid[biteTarget].isVictim = true
		raid[playerName].yourIcon = nil
		raid[playerName].yourIcon = self:GetRaidIcon()
		local message = "---->> "..format(L["Your bite target %s. Please bite -->> "], reason)..strupper(biteTarget).."  "
		if raid[playerName].yourIcon then
			marks[raid[playerName].yourIcon] = biteTarget
			message = message.."{rt"..raid[playerName].yourIcon.."} "
			message = message.."{rt"..raid[playerName].yourIcon.."} "
			message = message.."{rt"..raid[playerName].yourIcon.."} "
		else
			message = message..L["(No raid mark)"]
		end
  SendChatMessage(message, "WHISPER", nil, playerName)
  SendChatMessage(format(L["%s is coming to bite you!"], playerName), "WHISPER", nil, biteTarget)
  if self.db.profile.raidWarn and GetTime() - lastRaidWarning >= self.db.profile.raidWarnThrottle then
			lastRaidWarning = GetTime()
			SendChatMessage(L["One or more bite targets died or were disconnected. Check for new assignments."], "RAID_WARNING")
  end
	else
		SendChatMessage(format(L["Your bite target %s. There is no bite target left for you!"], reason), "WHISPER", nil, playerName)
 end
end


function Vamp:GetPlayerType(playerName)
	for k, v in pairs(self.db.profile.melee) do
		if playerName == v then return "melee" end
	end
	for k, v in pairs(self.db.profile.ranged) do
		if playerName == v then return "ranged" end
	end
	for k, v in pairs(self.db.profile.healers) do
		if playerName == v then return "healer" end
	end
	for k, v in pairs(self.db.profile.tanks) do
		if playerName == v then return "tank" end
	end
	return "unlisted"
end


function Vamp:AnyMeleeBit()
	for k, v in pairs(self.db.profile.melee) do
		if raid[v] and raid[v].group <= 5 and raid[v].isVampire and not UnitIsDead(v) and UnitIsConnected(v) then return true end
	end
	return false
end


function Vamp:AnyRangedBit()
	for k, v in pairs(self.db.profile.ranged) do
		if raid[v] and raid[v].group <= 5 and raid[v].isVampire and not UnitIsDead(v) and UnitIsConnected(v) then return true end
	end
	return false
end


function Vamp:AnyHealerBit()
	for k, v in pairs(self.db.profile.healers) do
		if raid[v] and raid[v].group <= 5 and raid[v].isVampire and not UnitIsDead(v) and UnitIsConnected(v) then return true end
	end
	return false
end


function Vamp:GetRaidIcon()
	for i = 1, 8 do
		local used = false
		for k, v in pairs(raid) do
			if v.group <= 5 and v.yourIcon and v.yourIcon == i then used = true end
		end
		if not used then return i end
	end
	return nil
end


function Vamp:FindTarget(type)
	if type == "unlisted" then
		for k, v in pairs(raid) do
			if self:GetPlayerType(k) == "unlisted" and self:IsValidTarget(k) then
				--self:Print("FindTarget called for type "..type..". Returning "..k)
				return k
			end
		end
	else
		for i = 1, LIST_SIZE do
			if self.db.profile[type][i] and self.db.profile[type][i] ~= "" and self:IsValidTarget(self.db.profile[type][i]) then
				--self:Print("FindTarget called for type "..type..". Returning "..self.db.profile[type][i])
				return self.db.profile[type][i]
			end
		end
	end
	--self:Print("FindTarget called for type "..type..". Returning nil")
	return nil
end


function Vamp:IsValidTarget(name)
	if raid[name] and raid[name].group <= 5 and not raid[name].isVampire and not raid[name].isVictim and not UnitIsDead(name) and UnitIsConnected(name) and not raid[name].mindControlled then
		--self:Print("IsValidTarget checking "..name..", result true")
		return true
	else
		--self:Print("IsValidTarget checking "..name..", result false")
		return false
	end
end


function Vamp:GetBiteTarget(playerName)
 -- Check which type of player is doing the biting and decide which type they should bite
 local playerType = self:GetPlayerType(playerName)
 local desiredVictimType = "melee"
 if playerType == "healer" then
		if self:AnyMeleeBit() then desiredVictimType = "ranged" else desiredVictimType = "melee" end
 elseif playerType == "tank" then desiredVictimType = "melee"
 elseif playerType == "ranged" then
		if self:AnyMeleeBit() then desiredVictimType = "ranged" else desiredVictimType = "melee" end
 elseif playerType == "melee" then
		if self:AnyRangedBit() or self:AnyHealerBit() then desiredVictimType = "melee" else desiredVictimType = "ranged" end
 end
 
 --self:Print("player type is "..playerType..", and biting a player type "..desiredVictimType)

	-- Get a target
	if desiredVictimType == "melee" then
		local target = self:FindTarget("melee")
		if target then return target end
		target = self:FindTarget("ranged")
		if target then return target end
		if self.db.profile.tanks[2] and self.db.profile.tanks[2] ~= "" and self:IsValidTarget(self.db.profile.tanks[2]) then return self.db.profile.tanks[2] end
		if self.db.profile.biteUnlisted then
			target = self:FindTarget("unlisted")
			if target then return target end
		end
		target = self:FindTarget("healers")
		if target then return target end
		if self.db.profile.tanks[1] and self.db.profile.tanks[1] ~= "" and self:IsValidTarget(self.db.profile.tanks[1]) then return self.db.profile.tanks[1] end
		
	else
		local target = self:FindTarget("ranged")
		if target then return target end
		target = self:FindTarget("melee")
		if target then return target end
		if self.db.profile.tanks[2] and self.db.profile.tanks[2] ~= "" and self:IsValidTarget(self.db.profile.tanks[2]) then return self.db.profile.tanks[2] end
		if self.db.profile.biteUnlisted then
			target = self:FindTarget("unlisted")
			if target then return target end
		end
		target = self:FindTarget("healers")
		if target then return target end
		if self.db.profile.tanks[1] and self.db.profile.tanks[1] ~= "" and self:IsValidTarget(self.db.profile.tanks[1]) then return self.db.profile.tanks[1] end

	end
	
	-- Couldn't find a target
	return nil
end


function Vamp:OnCombatEvent(event, timestamp, eventType, srcGuid, srcName, srcFlags, dstGuid, dstName, dstFlags, ...)
	if isEnabled and UnitInRaid("player") then
	
------[[ AURAS ]] ---------------------------------------------------------------------------------------------------------------------
		if eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH" then
			local playerName = dstName
			if raid[playerName] then
				local spellId, spellName, spellSchool, auraType = select(1, ...)
				
				if spellName == vampireDebuff or spellId == testDebuff then
				
					-- This player was just bit
					raid[playerName].isVampire = true
					raid[playerName].isBiting = false
	    
					-- Clear any old bite assignments
					if raid[playerName].yourVictim then
						raid[raid[playerName].yourVictim].isVictim = false
					end
	    
					--self:Print(playerName.." just got bit. We'd like him/her to bite a "..desiredVictimType.." player now.")
	    
					-- Find a bite target and notify them
					local biteTarget = self:GetBiteTarget(playerName)
					if biteTarget then
						raid[playerName].yourVictim = biteTarget
						raid[biteTarget].isVictim = true
						raid[playerName].yourIcon = nil
						raid[playerName].yourIcon = self:GetRaidIcon()
						local message = "---->> "..strupper(biteTarget).."  "
						if raid[playerName].yourIcon then
							marks[raid[playerName].yourIcon] = biteTarget
							message = message.."{rt"..raid[playerName].yourIcon.."} "
							message = message.."{rt"..raid[playerName].yourIcon.."} "
							message = message.."{rt"..raid[playerName].yourIcon.."} "
						else
							message = message..L["(No raid mark)"]
						end
						SendChatMessage(message, "WHISPER", nil, playerName)
						SendChatMessage(format(L["%s is coming to bite you!"], playerName), "WHISPER", nil, biteTarget)
					else
						SendChatMessage(L["No bite target left for you!"], "WHISPER", nil, playerName)
					end
		
		
				elseif spellName == vampireFrenzyDebuff then

					-- Player is in frenzy mode, they must bite someone
					raid[playerName].isVampire = false
					raid[playerName].isBiting = true
					raid[playerName].yourIcon = nil
					raid[playerName].yourVictim = nil

				
				elseif spellName == mindControlledDebuff then
					-- Player was MCed
					raid[playerName].mindControlled = true
					raid[playerName].yourIcon = nil
					raid[playerName].yourVictim = nil

				end
				
			end
			

	------[[ DEATH ]] ---------------------------------------------------------------------------------------------------------------------

		elseif eventType == "UNIT_DIED" and isEnabled and self.inCombat and UnitInRaid("player") then

			if raid[dstName] and raid[dstName].group <= 5 then
			
				-- Reset the dead player's settings
				raid[dstName].isVampire = false
				raid[dstName].isBiting = false
				raid[dstName].yourIcon = nil
				raid[dstName].yourVictim = nil
				raid[dstName].mindControlled = false

				-- See if the dead person was a bite target
				if raid[dstName].isVictim then
					raid[dstName].isVictim = nil
					
					-- Find out who was biting them and give them a new assignment
					for k, v in pairs(raid) do
						if v.yourVictim == dstName then
							self:ReassignTarget(k, dstName, L["died"])
							break
						end
					end
				end
			end
			
		end -- eventType
	end -- addon enabled, etc.
end

